import os
from dotenv import load_dotenv
from uuid import uuid4
from apify import Actor
from .constants import *

def env(_opt, _name, _task, _storage, _gpt=None, _start=None, _end=None):
    try:
        load_dotenv()
        global input_name
        global task_count
        global input_start
        global input_end
        global openai_api_key
        global storage_name
        global gpt_model
        input_name = _name
        task_count = _task
        storage_name = _storage
        if _opt == AF_CATALOG:
            input_start = _start
            input_end = _end
        if _opt == AF_GOLD_S01:
            openai_api_key = os.environ['OPENAI_API_KEY']
            gpt_model = _gpt
        if _opt == AF_GOLD_S02:
            openai_api_key = os.environ['OPENAI_API_KEY']
            gpt_model = _gpt
            # print(f"[API KEY]\n{os.environ['OPENAI_API_KEY'][:-25]}" + "*" * 25)
    except Exception as err:
        # raise InternalLlmError(err)
        raise

async def check_input(_opt):
    try:
        global input_name
        global input_start
        global input_end
        global task_count
        global openai_api_key
        global gpt_model
        actor_input = await Actor.get_input()
        cmd = actor_input.get(AF_CMD)
        if cmd == AF_CMD_INPUT:
            input_name = actor_input.get('name')
            task_count = actor_input.get('task')
            if _opt == AF_CATALOG:
                input_start = actor_input.get('start')
                input_end = actor_input.get('end')
            if _opt == AF_GOLD_S01:
                openai_api_key = actor_input.get('openai-api')
                gpt_model = GPT_MODEL
                # print(f"[API KEY]\n{os.environ['OPENAI_API_KEY'][:-25]}" + "*" * 25)
            if _opt == AF_GOLD_S02:
                openai_api_key = actor_input.get('openai-api')
                gpt_model = GPT_MODEL
            return True
        elif cmd == AF_CMD_UPDATE:
            if _opt == AF_CATALOG:
                scraperdata = actor_input.get('scraper')
                kv_store = await Actor.open_key_value_store(name=SX_KV_CONFIG)
                await kv_store.set_value(f'{SX_SCRAPER}{SX_EXT_YAML}', scraperdata)
            return False
    except Exception as err:
        # raise InternalLlmError(err)
        raise


#################################################################
# configure
#################################################################
SX_STORAGE_BASE = 'techlab'                 # '/Users/eastjoo/works/scraper'
SX_STORAGE_NAME = 'sx'

SX_FILE_SYTEM = 'file'
SX_APIFY_SYSTEM = 'apify'
SX_STORAGE_TYPE = SX_APIFY_SYSTEM

GPT_4o = 'gpt-4o-2024-08-06'                # 128,000 tokens / 16,384 tokens
GPT_4o_mini = 'gpt-4o-mini-2024-07-18'      # 128,000 tokens / 16,384 tokens
GPT_MODEL = GPT_4o_mini

SX_TASK_TIMEOUT = 180                       # task timeout
SX_MAX_AGENCY = 20                          # 신문사 length

SX_CATALOG_SIZE = 25
SX_BRONZE_SIZE = 20
SX_SILVER_SIZE = 30
SX_GOLD_SIZE = 30
SX_MART_01_SIZE = 10

SX_SQLDB_DIR = 'sqldb'

#################################################################
# constant
#################################################################
SX_VER = '1'

PROMPTS_DIR = 'prompts'                     # ssak3/prompt
PROMPT_GOLD_01 = 'gold_01'
PROMPT_GOLD_02_01 = 'gold_02_01'
PROMPT_GOLD_02_02 = 'gold_02_02'
PROMPT_MART_01_01 = 'mart_01_01'

# apify storage schema
AF_CATALOG = 'catalog'
AF_INDEX = 'index'
AF_BRONZE = 'bronze'
AF_SILVER = 'silver'
AF_GOLD_S01 = 'gold01'
AF_GOLD_S02 = 'gold02'
AF_MART_S01 = 'mart01'

# apify file name
AF_QRY_HDR = 'q'
AF_IDX_HDR = 'news-index-'
AF_TMP_HDR = '-'

# file storage schema
SX_CATALOG = '01.catalog'
SX_BRONZE = '02.bronze'
SX_BRONZE_RAW = '01.rawdata'
SX_BRONZE_FILTER = '02.filtereddata'
SX_SILVER = '03.silver'
SX_GOLD = '04.gold'
SX_GOLD_STEP01 = '01.step'
SX_GOLD_STEP02 = '02.step'

# file name
SX_QRY_HDR = 'query='
SX_IDX_HDR = 'news_index_'
SX_RAW_HDR = 'raw_'
SX_FLT_HDR = 'filter_'
SX_PUR_HDR = 'pur_'
SX_LLM_HDR = 'llm_'
SX_TMP_HDR = '_'
SX_S01_HDR = 's01_'
SX_S02_HDR = 's02_'
SX_M01_HDR = 'm01_'
SX_KEY_YEAR = 'year='
SX_KEY_MONTH = 'month='
SX_SCRAPER_NEWS = 'topic=news'
AF_SCRAPER_NEWS = 'news'

# logging
SX_LOGS = 'logs'
SX_LOG_CATALOG = 'sc'
SX_LOG_BRONZE = 'sb'
SX_LOG_SILVER = 'ss'
SX_LOG_GOLD_01 = 'sg01'
SX_LOG_GOLD_02 = 'sg02'
SX_LOG_MART_01 = 'sm01'

# internal
SX_SCRAPER = 'scraper'
SX_EXT_YAML = '.yaml'
SX_EXT_JSON = '.json'
SX_KV_CONFIG = f'{SX_STORAGE_BASE}-{SX_STORAGE_NAME}-{AF_SCRAPER_NEWS}-config'

AF_CMD = 'cmd'
AF_CMD_INPUT = 'input'
AF_CMD_UPDATE = 'update'

# __all__ = ['SX_GOLD_SIZE', SX_GOLD_SIZE]


import hashlib
import yaml
from apify import Actor
from .constants import *
from .utils import elapsed, prompt_template_yaml

# opt 종류
QUERY = 'query'
FIX = 'fix'
PROMPT = 'prompt'
KEYWORDS = 'keywords'
DESCRIPTION = 'description'
def get(_name, _opt):
    datadict = prompt_template_yaml(SX_SCRAPER)
    for entry in datadict:
        if entry.get('name') == _name:
            if _opt == FIX:
                return fix_query(entry[QUERY])
            return entry[_opt]
    return None

# 'Hello | World + 한글 - "Test"'
def fix_query(_naver_query):
    hash_bytes = hashlib.sha256(_naver_query.encode()).digest()
    out =  bytes(a ^ b ^ c ^ d for a, b, c, d in zip(hash_bytes[:8], hash_bytes[8:16], hash_bytes[16:24], hash_bytes[24:]))
    return out.hex()

async def getkv(_name, _opt):
    kv_store = await Actor.open_key_value_store(name=SX_KV_CONFIG)
    begintrx = await kv_store.get_value(f'{SX_SCRAPER}{SX_EXT_YAML}') or None
    if not begintrx:
        return get(_name, _opt)
    datadict = yaml.load(begintrx, Loader=yaml.FullLoader)
    for entry in datadict:
        if entry.get('name') == _name:
            if _opt == FIX:
                return fix_query(entry[QUERY])
            return entry[_opt]
    return None